import requests

from codecs import open
from requests.exceptions import MissingSchema, InvalidSchema, InvalidURL

from .parser import SynParser

def load_synonyms(path):
  try:
    r = requests.get(path)
    content = r.text
  except (MissingSchema, InvalidSchema, InvalidURL):
    try:
      with open(path, encoding='utf-8') as fp:
        content = fp.read()
    except (OSError, IOError):
      raise TypeError('Invalid Path: "{0}". Ensure it is either a URL or Correct FS Path.'.format(path))

  return SynParser.get_mapping(content)



import hues
import collections

from .constants import LEVELS


ReportMessage = collections.namedtuple('ReportMessage', ['level', 'msg', 'line_nb'])


def print_reports(reports, verbose=False):
  '''Utility function for rendering SynParser report.'''
  errored = False
  for report in reports:
    if report.level == LEVELS['ok']:
      if verbose:
        hues.log(report.line_nb, report.msg, time=False)
      continue
    elif report.level == LEVELS['warn']:
      hues.log(report.line_nb, report.msg, time=False, warn=True)
    elif report.level == LEVELS['error']:
      hues.log(report.line_nb, report.msg, time=False, error=True)
      errored = True
  if not errored:
    hues.log('Valid Synonyms', time=False, success=True)
  return errored


def validate_report(reports):
  '''Check if the parser reports any error.'''
  for report in reports:
    if report.level == LEVELS['error']:
      return False
  return True



import pytest
import os

from es_synonyms import parser, SynParser, load_synonyms
from es_synonyms import exceptions

def test_parse_empty_line():
  assert parser.empty_line(' ') is True
  assert parser.empty_line('\n') is True
  assert parser.empty_line('42') is None

def test_parse_comment_line():
  assert parser.comment_line('# 42') is True
  assert parser.comment_line('42') is None

def test_parse_mapping_line():
  assert parser.mapping_line('42') is True
  assert parser.mapping_line('42, 44') is True
  assert parser.mapping_line('a => b') is True
  assert parser.mapping_line('a, b => c, d') is True
  assert parser.mapping_line('a, b => c') is True

def test_parse_mapping_line_trailing_comma():
  with pytest.raises(exceptions.ParserError):
    assert parser.mapping_line('a, => b')

  with pytest.raises(exceptions.ParserError):
    assert parser.mapping_line('a, b => b, c,')

  with pytest.raises(exceptions.ParserError):
    assert parser.mapping_line('a,')

def test_parse_mapping_line_unbalanced_tokens():
  with pytest.raises(exceptions.ParserWarning):
    parser.mapping_line('a, b => c, d, e')

def test_parse_mapping_line_multiple_map():
  with pytest.raises(exceptions.ParserError):
    parser.mapping_line('a, b => c => d, e')

def test_parser_class__parse():
  content = '''
  hello, world
  bye, world
  '''
  SynParser.parse(content)

def test_parser_class__validate():
  valid = '''
  hello, world
  '''
  invalid = '''
  hello => world => world
  '''
  assert SynParser.validate(valid) is None
  with pytest.raises(exceptions.InvalidSynonyms):
    SynParser.validate(invalid)

def test_utils_load_file():
  syns = os.path.join(os.path.dirname(__file__), '../data/be-ae.synonyms')
  valid_path = os.path.abspath(syns)

  assert load_synonyms(valid_path)

def test_utils_load_incorrect_file():
  with pytest.raises(TypeError):
    load_synonyms('file://file_url_wont_work')

  with pytest.raises(TypeError):
    load_synonyms('weird_strings')

  with pytest.raises(TypeError):
    load_synonyms('./noo/nothing/here')



from .exceptions import ParserError, ParserWarning, InvalidSynonyms
from .constants import OPERATORS, LEVELS
from .report import ReportMessage, validate_report

def empty_line(line):
  '''Empty lines are ignored by Solr.'''
  if len(line.strip()) == 0:
    return True

def comment_line(line):
  '''Comments start with a pound.'''
  if line.startswith(OPERATORS['comment']):
    return True

def mapping_line(line):
  '''A mapping should contain single `=>` and may contain several `,`.'''
  token_empty = lambda x: len(x.strip()) == 0
  all_tokens = []

  if OPERATORS['map'] in line:
    for l in line.split(OPERATORS['map']):
      all_tokens += l.split(OPERATORS['delimiter'])
  else:
    all_tokens = line.split(OPERATORS['delimiter'])

  if any(map(token_empty, all_tokens)):
    raise ParserError('Invalid mapping. Check for spaces, and trailing commas.')

  if OPERATORS['map'] in line:
    if not line.count(OPERATORS['map']) == 1:
      raise ParserError('Cannot define multiple mapping in single line')

    lhs, rhs = line.split(OPERATORS['map'])

    tokens_l = lhs.split(OPERATORS['delimiter'])
    tokens_r = rhs.split(OPERATORS['delimiter'])

    if not len(tokens_l) == len(tokens_r) and not len(tokens_r) == 1:
      raise ParserWarning('Unbalanced tokens in mapping')

  return True


class SynParser(object):
  '''Parse Solr Synonyms'''
  hooks = [empty_line, comment_line, mapping_line]

  @classmethod
  def parse(cls, content=None, filename=None):
    if content is not None:
      return cls.parse_lines(content.splitlines())
    elif filename is not None:
      with open(filename, 'r') as fp:
        return cls.parse_lines(fp)
    else:
      raise TypeError('Either content or filename is required')

  @classmethod
  def parse_lines(cls, lines):
    '''Parse a list of lines.'''
    reports = []
    for line_nb, line in enumerate(lines):
      response = cls.parse_line(line, line_nb)
      reports.append(response)
    return reports

  @classmethod
  def parse_line(cls, line, line_nb):
    '''Parse a single line.'''
    for hook in cls.hooks:
      try:
        resp = hook(line)
        if resp is True:
          return ReportMessage(LEVELS['ok'], 'Ok', line_nb + 1)
      except ParserWarning as e:
        return ReportMessage(LEVELS['warn'], str(e), line_nb + 1)
      except ParserError as e:
        return ReportMessage(LEVELS['error'], str(e), line_nb + 1)

    return ReportMessage(LEVELS['warn'], 'Undefined parser state', line_nb + 1)

  @classmethod
  def validate(cls, content):
    report = cls.parse(content)
    if not validate_report(report):
      raise InvalidSynonyms('The synonyms file is invalid.')

  @classmethod
  def get_mapping(cls, content):
    try:
      cls.validate(content)
    except InvalidSynonyms as e:
      raise e
    else:
      mappings = []
      for mline in content.splitlines():
        l = mline.strip()
        if len(l) and not l.startswith(OPERATORS['comment']):
          mappings.append(l)
      return mappings



import argparse
import sys
import hues

from .parser import SynParser
from .report import print_reports


def cli():
  cli = argparse.ArgumentParser(description='Parse and validate Solr Synonyms file.')
  cli.add_argument('file', help='Solr Synonyms file', nargs='+')
  cli.add_argument('-v', '--verbose', action='store_true', help='Print a verbose report')

  args = cli.parse_args()

  status = 0
  for fname in args.file:
    hues.info('Validating', fname)
    reports = SynParser.parse(filename=fname)
    error = print_reports(reports, args.verbose)
    if error:
      status += 1

  sys.exit(status)



ParserException = type('ParserException', (Exception,), {})
ParserWarning = type('ParserWarning', (ParserException,), {})
ParserError = type('ParserError', (ParserException,), {})
InvalidSynonyms = type('InvalidSynonyms', (ParserException,), {})



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import re
import sys

from os import path
from codecs import open
from setuptools import setup

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as fp:
  long_description = fp.read()

with open(path.join(here, 'es_synonyms', '__init__.py'), encoding='utf-8') as fp:
  rex = r'^__version__ = \((\d+?), (\d+?), (\d+?)\)$'
  vtp = re.search(rex, fp.read(), re.M).groups()
  __version__ = '.'.join(vtp)

install_requires = ('hues', 'requests',)
setup_requires = ('pytest-runner',)
test_requirements = ['pytest', 'pytest-sugar']

if sys.argv[-1] == 'publish':
    try:
      import wheel
      print("Wheel version: ", wheel.__version__)
    except ImportError:
      print('Wheel library missing. Please run "pip install wheel"')
      sys.exit()
    os.system('python setup.py sdist upload')
    os.system('python setup.py bdist_wheel upload')
    sys.exit()

if sys.argv[-1] == 'tag':
    print("Tagging the version on git:")
    os.system("git tag -a %s -m 'version %s'" % (__version__, __version__))
    os.system("git push --tags")
    sys.exit()

setup(
  name='elasticsearch-synonym-toolkit',
  version=__version__,
  description="""Toolkit for Elasticsearch Synonym files.""",
  long_description=long_description,
  author='Prashant Sinha',
  author_email='prashant+git@noop.pw',
  url='https://github.com/prashnts/elasticsearch-synonyms',
  download_url='https://github.com/prashnts/elasticsearch-synonym/tarball/' + __version__,
  packages=[
    'es_synonyms',
  ],
  include_package_data=True,
  install_requires=install_requires,
  setup_requires=setup_requires,
  tests_require=test_requirements,
  entry_points={
    'console_scripts': ['es-synlint=es_synonyms.synlint:cli'],
  },
  license="MIT",
  zip_safe=False,
  keywords='elasticsearch-synonym-toolkit',
  classifiers=[
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',
    'Natural Language :: English',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.2',
    'Programming Language :: Python :: 3.3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
  ],
)
